//
//  myToken.cpp
//

/*  This file should be accompanied by DISCLAIMER.TXT stating disclaimers */

#include "myToken.h"
#include "string.h"
#include "myPanic.h"
#include <stdio.h>
#include "P.h"

void ANTLRToken::init() {
  col=0;
  pText=0;
}

ANTLRToken::ANTLRToken() {
  init();
  setLine(0);
}

ANTLRToken::~ANTLRToken() {
////   dumpNode("Token Destructor");
   delete [] pText;
   pText=0;
}

ANTLRToken::ANTLRToken(ANTLRTokenType tokenTypeNew) :
    SimpleToken(tokenTypeNew) {
   setLine(0);
   init();
}

ANTLRToken::ANTLRToken(ANTLRTokenType   tokenTypeNew,
               ANTLRChar *  textNew,   
               int      lineNew) :
    SimpleToken(tokenTypeNew) {
   setLine(lineNew);
   init();
   setText(textNew);
}         

ANTLRToken::ANTLRToken(const ANTLRToken & from) :
    SimpleToken(from) {
   init();
   col=from.col;
   setText(from.pText);
};  

//  create new copy of text - not just another reference to existing text

ANTLRToken & ANTLRToken::operator = (const ANTLRToken & from) {
   if (this != &from) {
     col=from.col;
     setText(from.pText);
   };
   return *this;
}      

//  create new copy of text - not just another reference to existing text

void ANTLRToken::setText(const ANTLRChar *s) {

   const ANTLRChar *p;

   if (pText != 0) {
      delete [] pText;
      pText=0;
   };
   if (s != 0) {
//
//  skip leading spaces, newlines, tabs
//  this results in null pointers for strings which are just spaces & newlines
//
     for (p=s;*p != 0;p++) {
        if (*p != ' '  &&
            *p != '\n' &&
            *p != '\t') break;
     };
     if (*p != 0) {
       pText=new char [strlen(p)+1];
       if (pText == 0) myPanic ("ANTLRToken::setText strdup failed");
       strcpy(pText,p);
     };
   };
}

ANTLRAbstractToken * ANTLRToken::makeToken(ANTLRTokenType   tokenType,
                           ANTLRChar *      text,
                           int          line) {
   return new ANTLRToken(tokenType,text,line);
}

void ANTLRToken::dumpNode(const char * s) {

   char * theText;

   if (s != 0) {printf("%s ",s);};
   if (getType() == Eof) {
     printf("TokenType \"EOF\" Token # %d\n",serial);
   } else {
     if (pText == 0) {
        theText="";
     } else if (strcmp(pText,"\n") == 0) {
    theText="NL";
     } else {
    theText=pText;
     }; 
     printf("TokenType (%s) Text (%s) Token # %d  Line=%d  Col=%d\n",
        P::tokenName(getType()),
        theText,
        serial,     
        getLine(),
        col);
   };
}
